package PipSqueek::Plugin::BotAdmin;
use base qw(PipSqueek::Plugin);

sub plugin_initialize
{
	my $self = shift;

	$self->plugin_handlers([
		'multi_reload',
		'multi_say',
		'multi_act',
		'multi_raw',
		'multi_partchan',
		'multi_joinchan',
		'multi_cyclechan',
		'multi_mode',
		'multi_botlevel',
		'multi_shutdown',
		'multi_mergeuser',
		'multi_mode',
	]);
}

sub multi_reload
{
	my ($self,$message) = @_;

	$self->client()->kernel()->call( $self->client()->SESSION_ID(),'plugins_load' );
	return $self->respond( $message, "Bot reloaded" );
}


sub multi_say
{
	my ($self,$message) = @_;
	my ($channel,$text) = $message->command_input() =~
		m/^(?:([#&+!].*?)\s+)?(.*)$/;
	$channel ||= $self->config()->server_channel();
	$self->client()->privmsg( $channel, $text );
}


sub multi_act
{
	my ($self,$message) = @_;
	my ($channel,$text) = $message->command_input() =~
		m/^(?:([#&+!].*?)\s+)?(.*)$/;
	$channel ||= $self->config()->server_channel();
	$self->client()->ctcp( $channel, "ACTION $text" );
}


sub multi_raw
{
	my ($self,$message) = @_;
	my ($raw) = $message->command_input() || return;
	$self->client()->sl( $raw );
}

sub multi_partchan
{
	my ($self,$message) = @_;
	my $channel = $message->channel() || $message->command_input() || $self->config()->server_channel();
	$self->client()->part( $channel );
}


sub multi_joinchan
{
	my ($self,$message) = @_;
	my $channel = $message->command_input() || $self->config()->server_channel();
	$self->client()->join( $channel );
}


sub multi_cyclechan
{
	my ($self,$message) = @_;
	my $channel = $message->channel() || $message->command_input() || $self->config()->server_channel();
	$self->client()->part( $channel );
	$self->client()->join( $channel );
}


sub multi_shutdown
{
	my ($self,$message) = @_;

	$self->client()->yield( 'session_shutdown' );
}
		
sub multi_botlevel
{
	my ($self,$message) = @_;
	my ($name,$level) = split( /\s+/, $message->command_input() );

	if( $name && defined($level) )
	{
		my $user = $self->search_user( $name );

		unless( $user )
		{
			$self->respond( $message, "That user does not exist" );
			return;
		}
		
		$user->{'cmd_level'} = $level;

		$self->update_user( $user );
	}
}

sub multi_mergeuser
{
	my ($self,$message) = @_;
	my ($u1name,$u2name) = split(/\s+/,$message->command_input());

	unless( defined($u1name) && defined($u2name) )
	{
		$self->respond( $message, "See !help mergeuser" );
		return;
	}

	my $user1 = $self->search_user($u1name);
	my $user2 = $self->search_user($u2name);

	unless( defined($user1) && $user1->{'id'} )
	{
		$self->respond( $message, "User not found: '$u1name'" );
		return;
	}

	unless( defined($user2) && $user2->{'id'} )
	{
		$self->respond( $message, "User not found: '$u2name'" );
		return;
	}

	if( $user1->{'id'} == $user2->{'id'} )
	{
		$self->respond( $message, 
			"You can't merge a user with himself" );
		return;
	}
	
	
	$self->client()->yield( 'pipsqueek_mergeuser', $message, 
				$user1, $user2 );

	$self->respond( $message, "'mergeuser' event dispatched.  It could take a few seconds for all data to be updated" );

	return;
}


sub multi_mode
{
	my ($self,$message) = @_;
	my @args = split( /\s/, $message->command_input() );

	$self->client()->mode( @args );
}


1;


__END__
