package PipSqueek::Plugin::ChanAdmin;
use base qw(PipSqueek::Plugin);
use File::Spec::Functions qw(catfile);
use strict;
###############################################################################
#Modes
#1		Auto Voice
#2		Auto Halfop
#4		Auto Op
#6		Auto Protect
#Possible Combinations
#1		Voice
#2		Halfop
#3		Voice and Halfop
#4		Op 
#5		Voice+Op
#6		Protect
#7		Voice+Protect
#8		Halfop+Protect
#9		Voice+HalfOp+Protect
#10		Op+Protect
#11		Voice+Op+Protect
###############################################################################
sub plugin_initialize
{
	my $self = shift;
		
	$self->plugin_handlers({
		'multi_setlevel'	=> 'multi_setlevel',
		'multi_topic'		=> 'multi_topic',
		'multi_voice' 		=> 'multi_voice',
		'multi_devoice' 	=> 'multi_devoice',
		'multi_halfop' 		=> 'multi_halfop',
		'multi_op' 			=> 'multi_op',
		'multi_dehalfop'	=> 'multi_dehalfop',
		'multi_deop' 		=> 'multi_deop',
		'multi_protect' 	=> 'multi_protect',
		'multi_deprotect'	=> 'multi_deprotect',
		'multi_kick' 		=> 'multi_kick',
		'multi_invite'		=> 'multi_invite',
		'multi_ban'			=> 'multi_ban',
		'multi_kban'		=> 'multi_kban',		
		'multi_+avoice'   	=> 'multi_add_autovoice',
		'multi_-avoice'   	=> 'multi_del_autovoice',
		'multi_+aop'      	=> 'multi_add_autoop',
		'multi_-aop'      	=> 'multi_del_autoop',
		'multi_+ahalfop'  	=> 'multi_add_autohalfop',
		'multi_-ahalfop'  	=> 'multi_del_autohalfop',
		'multi_+aprotect' 	=> 'multi_add_autoprotect',
		'multi_-aprotect' 	=> 'multi_del_autoprotect',
		'multi_gr'		  	=> 'multi_getrecord',
		'multi_sr'		  	=> 'multi_setrecord',
		'multi_part'		=> 'multi_part',
		'multi_join'		=> 'multi_join',
		'multi_cycle'		=> 'multi_cycle'
		
	});

	my $schema = [
		[ 'id',			'INTEGER PRIMARY KEY' ],
		[ 'name',		'VARCHAR NOT NULL' ],
		[ 'ident',		'VARCHAR' ],
		[ 'host',		'VARCHAR' ],
		[ 'mode',		'INT NOT NULL DEFAULT 0' ],
		[ 'channel',	'VARCHAR NOT NULL' ],
		[ 'cmd_level',	'INT NOT NULL DEFAULT 10' ],
	];

	$self->dbi()->install_schema( 'auto', $schema );
	$self->_load_clevels_conf();
	
}
sub multi_part
{
	my ($self,$message) = @_;
	my $channel = $message->channel() || $message->command_input();
	if(!defined($channel))
	{
		$self->client()->privmsg($message->nick(),"Incorrect Parameter");
		return;
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->part( $channel );
	}
}
sub multi_join
{
	my ($self,$message) = @_;
	my $channel = $message->channel() || $message->command_input();
	if(!defined($channel))
	{
		$self->client()->privmsg($message->nick(),"Incorrect Parameter");
		return;
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->join( $channel );
	}
}
sub multi_cycle
{
	my ($self,$message) = @_;
	my $channel = $message->channel() || $message->command_input();
	if(!defined($channel))
	{
		$self->client()->privmsg($message->nick(),"Incorrect Parameter");
		return;
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->part( $channel );
		$self->client()->join( $channel );
	}
}
sub multi_topic
{
	my ($self,$message) = @_;
	my ($channel,$topic) = $message->command_input() =~
		m/^(?:([#&+!].*?)\s+)?(.*)$/;
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->topic( $channel, $topic );
	}
}

sub multi_voice
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '+v', $name );
	}
}

sub multi_devoice
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '-v', $name );
	}
}

sub multi_halfop
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '+h', $name );
	}
}

sub multi_dehalfop
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '-h', $name );
	}
}

sub multi_op
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '+o', $name );
	}
}

sub multi_deop
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '-o', $name );
	}
}

sub multi_protect
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '+a', $name );
	}
}

sub multi_deprotect
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->search_user($name );
	return unless $user;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '-a', $name );
	}
}

sub multi_kick
{
	my ($self,$message) = @_;
	my ($name,$channel,$msg) = $message->command_input() =~ 
		m/^(.+?)(?:\s+([#&+!].+?))?(?:\s+(.+))?$/;

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	return unless $name;
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->kick( $channel, $name, $msg );
	}
}

sub multi_ban
{
	my ($self,$message) = @_;
	my ($name,$channel,$mode) = split(/\s+/, $message->command_input());

	if( !$mode && $channel =~ /^[\d]$/ )
	{
		$mode = $channel;
		$channel = undef;
	}
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	$mode ||= $self->config()->default_ban_type() || 4;

	my $user = $self->search_user( $name );
	my ($ident,$host);

	if( $user ) {
		$ident = $user->{'ident'};
		$host  = $user->{'host'};
		$ident =~ s/^~//;
	} else {
		$mode = 1;
	}

	my $ban;
	foreach ( $mode )
	{
		   if( /1/ ) { $ban = qq(*$name!*\@*) }
		elsif( /2/ ) { $ban = qq(*!*$ident\@*) }
		elsif( /3/ ) { $ban = qq(*$name!*$ident\@*) }
		elsif( /4/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*!*\@$host);
		}
		elsif( /5/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*$name!*\@$host);
		}
		elsif( /6/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*!*$ident\@$host);
		}
		elsif( /7/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*$name!*$ident\@$host);
		}
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '+b', $ban );
	}
}

sub multi_invite
{
	my ($self,$message) = @_;
	my ($nickname,$channel) = split(/\s+/,$message->command_input());

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	if( !defined($nickname) || $nickname eq "" )
	{
		$self->respond( $message, "See !help invite" );
		return;
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->invite( $nickname, $channel );
	}
}

sub multi_kban
{
	my ($self,$message) = @_;
	my ($name,$channel,$mode) = split(/\s+/, $message->command_input());

	if( !$mode && $channel =~ /^[\d]$/ )
	{
		$mode = $channel;
		$channel = undef;
	}

	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	$mode ||= $self->config()->default_ban_type() || 4;

	my $user = $self->search_user( $name );

	my ($ident,$host);

	if( $user ) {
		$ident = $user->{'ident'};
		$host  = $user->{'host'};
		$ident =~ s/^~//;
	} else {
		$mode = 1;
	}
	
	my $ban;
	foreach ( $mode )
	{
		   if( /1/ ) { $ban = qq(*$name!*\@*) }
		elsif( /2/ ) { $ban = qq(*!*$ident\@*) }
		elsif( /3/ ) { $ban = qq(*$name!*$ident\@*) }
		elsif( /4/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*!*\@$host);
		}
		elsif( /5/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*$name!*\@$host);
		}
		elsif( /6/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*!*$ident\@$host);
		}
		elsif( /7/ ) {
			$host =~ s/^.*?\./*./;
			$ban = qq(*$name!*$ident\@$host);
		}
	}
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		$self->client()->mode( $channel, '-a', $name );
		$self->client()->kick( $channel, $name, "Get a Life");
		$self->client()->mode( $channel, '+b', $ban );
	}
}

sub multi_getrecord
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 

	my $auto = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $user = $self->search_user($message);
	
	$self->respond($message,"Mode = $auto->{'mode'} AND Level = $auto->{'cmd_level'} AND Identified = $user->{'identified'} AND Bot Level = $user->{'level'}");	
}

sub multi_setrecord
{
	my ($self,$message) = @_;
	my ($name,$level,$channel) = split(/\s+/, $message->command_input()); 
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	my $user = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	$user->{'mode'} = $level;
	$self->dbi()->update_record( 'auto',$user);
}
	
sub multi_add_autovoice
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input());
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
		
	my $user = $self->dbi()->select_record( 'users', { 'username' => $name});
    my ($ident,$host);

	if( $user )
	{
	$ident = $user->{'ident'};
	$host  = $user->{'host'};
	$ident =~ s/^~//;
	}
													
	unless( $name )
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp +avoice" );
		return;
	}
	my $avoice = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $oldmode=$avoice->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if( $avoice->{'id'} != 0 )
		{
			if($oldmode == 1 || $oldmode == 3 || $oldmode == 5 || $oldmode == 7 || $oldmode == 9 || $oldmode == 11)
			{
				$self->client()->privmsg( $message->nick(), "$avoice->{'name'} already autovoiced in $channel");
       			$self->client()->mode( $channel, '+v', $name );
				return;
			}
			else 
			{
				$avoice->{'mode'}=$oldmode+1;
				$self->dbi()->update_record('auto',$avoice);
				$self->client()->privmsg( $message->nick(), "Added $avoice->{'name'} to AutoVoice list of $channel" );
       			$self->client()->mode( $channel, '+v', $name );
				return;
			}
		}
		my $newmode=$oldmode+1;
		$avoice = $self->dbi()->create_record( 'auto',
		  	{	'ident' => $ident,
			    'host' => $host,
			    'name' => $name, 
			    'channel' => $channel,
			    'mode' => $newmode }
	  		);
	 
		$self->client()->privmsg( $message->nick(), "Added $avoice->{'name'} to AutoVoice list of $channel" );
		$self->client()->mode( $channel, '+v', $name );
	}
	return;
}
sub multi_del_autovoice
{
   	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	if(!defined($name) || !defined($channel))
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp -avoice" );
		return;
	}
	my $checked = (shift)->check_level(@_,$channel);
	my $avoice = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $oldmode=$avoice->{'mode'};
	if($checked != 0)
	{
		if ( !$avoice->{'name'})
		{
		$self->client()->privmsg( $message->nick(),"$name is not autovoiced in $channel");
		return;
		}
		if ($avoice->{'name'})
		{
			if($oldmode != 1 && $oldmode != 3 && $oldmode != 5 && $oldmode != 7 && $oldmode !=9 && $oldmode != 11) 
		        {
		               $self->client()->privmsg( $message->nick(), "$avoice->{'name'} is not autovoiced in $channel");
   	     		       $self->client()->mode( $channel, '-v', $name );
		               return;
		        }
			else	
			{
				$avoice->{'mode'} -= 1;
				if ($avoice->{'mode'}==0)
				{
					$self->dbi()->delete_record('auto', $avoice);
				}
		    }	
		$self->dbi()->update_record('auto', $avoice );
       	$self->client()->mode( $channel, '-v', $name );
		}
		$self->client()->privmsg( $message->nick(), "$avoice->{'name'} removed from avoice list of $channel" );
        $self->client()->mode( $channel, '-v', $name );
	}    
}						      

sub multi_add_autohalfop
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	
	my $user = $self->dbi()->select_record( 'users', { 'username' => $name});
	
    my ($ident,$host);

	if( $user )
	{
	$ident = $user->{'ident'};
	$host  = $user->{'host'};
	$ident =~ s/^~//;
	}
													
	unless( $name )
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp +ahalfop" );
		return;
	}
	my $ahalfop = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $oldmode=$ahalfop->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if( $ahalfop->{'id'} != 0)
		{
			if( $oldmode == 2 || $oldmode == 3 || $oldmode == 8 || $oldmode == 9)
			{
				$self->client()->privmsg( $message->nick(), "$ahalfop->{'name'} already autohalfoped");
        		$self->client()->mode( $channel, '+h', $name );
				return;
			}
			elsif ($oldmode == 4 || $oldmode == 5 || $oldmode == 10 || $oldmode == 11 )
			{	
				$self->client()->privmsg( $message->nick(), "$ahalfop->{'name'} removed from autoop and added to autohalfop of $channel");
				$ahalfop->{'mode'}= $oldmode-2;
				$self->dbi()->update_record('auto',$ahalfop);
				$self->client()->privmsg( $message->nick(), "Added $ahalfop->{'name'} to AutoHalfOp list of $channel");
        		$self->client()->mode( $channel, '-o', $name );
        		$self->client()->mode( $channel, '+h', $name );
				return;
			}
			else 
			{
				$ahalfop->{'mode'}=$oldmode+2;
				$self->dbi()->update_record('auto',$ahalfop);
				$self->client()->privmsg( $message->nick(), "Added $ahalfop->{'name'} to AutoHalfOp list of $channel");
        		$self->client()->mode( $channel, '+h', $name );
				return;
			}
		}
	
		my $newmode=$oldmode+2;
		my $ahalfop = $self->dbi()->create_record( 'auto',
			  { 'ident' => $ident,
			    'host' => $host,
			    'name' => $name, 
			    'channel' => $channel,
			    'mode' => $newmode }
		   );
		$self->client()->privmsg( $message->nick(), "Added $ahalfop->{'name'} to AutoHalfOp list of $channel");
        $self->client()->mode( $channel, '+h', $name );
	}
	return;
}
sub multi_del_autohalfop
{
   	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 	
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	if(!defined($name) || !defined($channel))
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp -ahalfop" );
		return;
	}

	my $ahalfop = $self->dbi()->select_record( 'auto',{ 'name' => $name ,'channel' => $channel } );
	my $oldmode=$ahalfop->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if ( !$ahalfop->{'name'})
		{
			$self->client()->privmsg( $message->nick(),"$name is not autohalfoped in $channel");
			return;
		}
		if ( $ahalfop->{'id'} != 0 )
		{
			if($oldmode != 2 && $oldmode != 8 && $oldmode != 9) 
		        {
	               $self->client()->privmsg( $message->nick(), "$ahalfop->{'name'} is not autohalfoped in $channel");
	   		       $self->client()->mode( $channel, '-h',$name );
	               return;
		        }
			else	
				{
					$ahalfop->{'mode'} = $oldmode-2;
					if ($ahalfop->{'mode'}==0)
					{
						$self->dbi()->delete_record('auto', $ahalfop);
					}
		     	}	
			$self->dbi()->update_record('auto', $ahalfop );
       		$self->client()->mode( $channel, '-h', $name );
		}
		$self->client()->privmsg( $message->nick(), "$ahalfop->{'name'} removed from ahalfop list in $channel" );
		$self->client()->mode( $channel, '-h', $name );
	}
}						      

sub multi_add_autoop
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	my $user = $self->dbi()->select_record( 'users', { 'username' => $name});
	
    my ($ident,$host);

	if( $user )
	{
	$ident = $user->{'ident'};
	$host  = $user->{'host'};
	$ident =~ s/^~//;
	}
													
	unless( $name )
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp +aop" );
		return;
	}
	my $aop = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $oldmode = $aop->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if( $aop->{'id'} != 0)
		{
			if( $oldmode == 4 || $oldmode == 5 || $oldmode == 10 || $oldmode == 11)
			{
				$self->client()->privmsg( $message->nick(), "$aop->{'name'} already autooped");
        		$self->client()->mode( $channel, '+o', $name );
				return;
			}
			elsif ($oldmode == 2 || $oldmode == 3 || $oldmode == 8 || $oldmode ==9)
			{	
				$self->client()->privmsg( $message->nick(), "$aop->{'name'} removed from autohalfop and added to autoop");
				$aop->{'mode'}= $oldmode+2;
				$self->dbi()->update_record('auto',$aop);
				$self->client()->privmsg( $message->nick(), "Added $aop->{'name'} to AutoOp list of $channel");
        		$self->client()->mode( $channel, '-h', $name );
        		$self->client()->mode( $channel, '+o', $name );
				return;
			}
			else 
			{
				$aop->{'mode'}=$oldmode+4;
				$self->dbi()->update_record('auto',$aop);
				$self->client()->privmsg( $message->nick(), "Added $aop->{'name'} to AutoOp list of $channel");
        		$self->client()->mode( $channel, '+o', $name );
				return;
			}	
		}
		my $newmode=$oldmode+4;
		my $aop = $self->dbi()->create_record( 'auto',
			  { 'ident' => $ident,
			    'host' => $host,
			    'name' => $name, 
			    'channel' => $channel,
			    'mode' => $newmode }
		   );
		$self->client()->privmsg( $message->nick(), "Added $aop->{'name'} to AutoOp list of $channel");
        $self->client()->mode( $channel, '+o', $name );
	}
	return;
}
sub multi_del_autoop
{
   	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 	
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	if(!defined($name) || !defined($channel))
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp -aop");
		return;
	}
	my $aop = $self->dbi()->select_record( 'auto',{ 'name' => $name ,'channel' => $channel } );
	my $oldmode=$aop->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if ( !$aop->{'name'})
		{
			$self->client()->privmsg( $message->nick(),"$name is not autooped in $channel");
			return;
		}
		if ( $aop->{'id'} != 0 )
		{
			if($oldmode != 4 && $oldmode != 5 && $oldmode != 10 && $oldmode != 11) 
			{
				$self->client()->privmsg( $message->nick(), "$aop->{'name'} is not autooped in $channel");
        		$self->client()->mode( $channel, '-o', $name );
		        return;
		    }
			else	
			{
				$aop->{'mode'} = $oldmode-4;
				if ($aop->{'mode'}==0)
				{
					$self->dbi()->delete_record('auto', $aop);
				}
	     	}	
			$self->dbi()->update_record('auto', $aop );
       		$self->client()->mode( $channel, '-o', $name );
		}
		$self->client()->privmsg( $message->nick(), "$aop->{'name'} removed from aop list of $channel" );
	    $self->client()->mode( $channel, '-o', $name );
	}
}						      

sub multi_add_autoprotect
{
	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 
	
	$channel ||= $message->channel();
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}
	
	my $user = $self->dbi()->select_record( 'users', { 'username' => $name});
	
    my ($ident,$host);

	if( $user )
	{
	$ident = $user->{'ident'};
	$host  = $user->{'host'};
	$ident =~ s/^~//;
	}
													
	unless( $name )
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp +aprotect" );
		return;
	}
	my $aprotect = $self->dbi()->select_record( 'auto', { 'name' => $name,'channel' => $channel});
	my $oldmode = $aprotect->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if( $aprotect->{'id'} != 0)
		{
			if( $oldmode >=7)
			{
				$self->client()->privmsg( $message->nick(), "$aprotect->{'name'} already in autoprotect list of $channel");
        		$self->client()->mode( $channel, '+a', $name );
				return;
			}
			else 
			{
				$aprotect->{'mode'}=$oldmode+6;
				$self->dbi()->update_record('auto',$aprotect);
				$self->client()->privmsg( $message->nick(), "Added $aprotect->{'name'} to AutoProtect list of $channel");
        		$self->client()->mode( $channel, '+a', $name );
				return;
			}
		}
		my $newmode=$oldmode+6;
		my $aprotect = $self->dbi()->create_record( 'auto',
			  { 'ident' => $ident,
			    'host' => $host,
			    'name' => $name, 
			    'channel' => $channel,
			    'mode' => $newmode }
		   );
		
		$self->client()->privmsg( $message->nick(), "Added $aprotect->{'name'} to AutoProtect list of $channel");
		$self->client()->mode( $channel, '+a', $name );
	}
	return;
}
sub multi_del_autoprotect
{
   	my ($self,$message) = @_;
	my ($name,$channel) = split(/\s+/, $message->command_input()); 	
	
	$channel ||= $message->channel();
	
	
	if(!defined($channel))
	{
		$channel = $self->config()->server_channel();
	}

	if(!defined($name) || !defined($channel))
	{
		$self->client()->privmsg( $message->nick(), "Use !chelp -aprotect" );
		return;
	}
	my $aprotect = undef;

	$aprotect = $self->dbi()->select_record( 'auto',{ 'name' => $name ,'channel' => $channel } );
	my $oldmode=$aprotect->{'mode'};
	my $checked = (shift)->check_level(@_,$channel);
	if($checked != 0)
	{
		if ( !$aprotect->{'name'})
		{
		$self->client()->privmsg( $message->nick(),"$name is not autoprotected in $channel");
		return;
		}
		if ( $aprotect->{'id'} != 0 )
		{
			if($oldmode != 6 && $oldmode != 7 && $oldmode != 8 && $oldmode != 9 && $oldmode != 10 && $oldmode != 11) 
		        {
	               $self->client()->privmsg( $message->nick(), "$aprotect->{'name'} is not autoprotected in $channel");
       		       $self->client()->mode( $channel, '-a', $name );
	               return;
		        }
			else	
			{
				$aprotect->{'mode'} = $oldmode-6;
				if ($aprotect->{'mode'}==0)
				{
					$self->dbi()->delete_record('auto', $aprotect);
				}
		     	}	
			$self->dbi()->update_record('auto', $aprotect );
   	    	$self->client()->mode( $channel, '-a', $name );
		}
		$self->client()->privmsg( $message->nick(), "$aprotect->{'name'} removed from aprotect list in $channel" );
        $self->client()->mode( $channel, '-a', $name );
	}

}				
sub multi_setlevel
{
	my ($self,$message) = @_;
	my ($name,$channel,$level) = split( /\s+/, $message->command_input() );
	my $cmdcheck = $self->dbi()->select_record( 'auto', { 'name' => $message->nick(),'channel' => $channel});
	my $checked = (shift)->check_level(@_);	
	
	my $user = $self->dbi()->select_record( 'users', { 'username' => $name});
    my ($ident,$host);
	
	if( $user )
	{
		$ident = $user->{'ident'};
		$host  = $user->{'host'};
		$ident =~ s/^~//;
	}

	if(!defined($name) || !defined($channel) || !defined($level))
		{
			$self->respond($message, "Incomplete Parameters");
			return;
		}
	if($level > 800)
	{
		$self->client()->privmsg($message->nick(),"You currently don't have the requisite permissions in $channel");
	}
	if($checked != 0)	
	{
		if( defined($name) && defined($level) && defined($channel))
		{
			my $access = $self->dbi()->select_record('auto',{'name' => $name, 'channel' => $channel});
			
			if(defined($access->{'id'}))
			{	
				$access->{'cmd_level'} = $level;
				$self->dbi()->update_record('auto',$access);
				return;
			}
			else
			{
				$self->dbi()->create_record('auto',{
							'name' => $name,
							'channel' => $channel,
							'host' => $host,
							'ident' => $ident,
							'cmd_level'=> $level,
				});
			return;
			}
		}
	}
}
sub check_level
{
	my ($self,$message,$channel) = @_;
	my $command = $message->command();
	my $user  = $self->search_user($message);
	my $clevel = $self->{'CLEVELS'}->{$command};

	my $name = $message->channel() || $channel;
	my $auto  = $self->dbi()->select_record('auto',{'name' => $name, 'channel' => $channel});

#	if($user->{'identified'} != 1)
#	{
#		$self->client()->privmsg( $message->nick(), "You currently currently not identified");
#		return 0;
#	}
#	else
#	{
		if ($user->{'cmd_level'} > $clevel)
		{
			return 1;
		}
		else
		{
			if( $auto->{'cmd_level'} >$clevel)
			{
				return 1;
			}
			else
			{ 
				$self->client()->privmsg( $message->nick(), "You currently don't have a the requisite access in $channel");
				return 0;
			}
		}
#	}
}		      
sub _load_clevels_conf
{
	my $self = shift;
	my $client = $self->client();

	my $loaded = 0;

	foreach my $dir ( $client->ROOTPATH(), $client->BASEPATH() )
	{
		my $file = catfile( $dir, '/etc/clevels.conf' );

		if( -e $file ) 
		{
			$self->_merge_clevels_conf_file( $file );
			$loaded = 1;
		}
	}

	unless( $loaded )
	{
		warn "Unable to find clevels.conf for " .
			$self->config()->server_channel() . "\n";
	}
}

sub _merge_clevels_conf_file
{
	my ($self,$file) = @_;

	open( my $fh, '<', $file )
		or die "Unable to open '$file': $!";
	my @lines = <$fh>;
	chomp(@lines);
	close( $fh );

	foreach my $line (grep(/=/,@lines))
	{
		my ($k,$v) = split(/=/,$line);
		$self->{'CLEVELS'}->{$k} = $v;
	}
}

1;
__END__
