package PipSqueek::Plugin::Misc;
use base 'PipSqueek::Plugin';
use strict;

# This plugin defines the basic operations of the pipsqueek bot

# things like command delegation, user registration/user management, 
# user authorization, and some ctcp replies are handled here

use File::Spec::Functions qw(catfile);


sub plugin_initialize
{
	my $self = shift;

	$self->plugin_handlers([
		'irc_join', # for user tracking
		'irc_part',
	]);

	# set up our database table for users
	my $schema = [
		[ 'id', 	'INTEGER PRIMARY KEY' ],
		[ 'user',	'VARCHAR NOT NULL' ],
		[ 'name',	'VARCHAR NOT NULL' ],
		[ 'join_time',	'TIMESTAMP' ],
	];

	$self->dbi()->install_schema( 'users_misc', $schema );
}


sub irc_join
{
	my ($self,$message) = @_;
	my $channel=$message->{'channel'};
 	my $prep=$self->search_or_create_user($message);
	my $name=$prep->{'nickname'};
	$prep->{'host'} =~ s/^.*?\./*./;
	my $user = "$name\!$prep->{'ident'}\@$prep->{'host'}";
	my $flyby = $self->dbi()->select_record('users_misc',{'LOWER(name)' => lc($name) });
	if($flyby->{'id'}!=0)
	{
		$flyby->{'join_time'}=time();
		$self->dbi()->update_record('users_misc',$flyby);
	}
	else
	{
	
		$self->dbi()->create_record('users_misc',
		{	'user' => $user, 			
			'name' => $prep->{'nickname'},
			'join_time'=> time()
			}
		);
	}

	my $autoimp = $self->dbi()->select_record('auto',{ 'name' => $name,'channel' => $channel});
if ($autoimp->{'mode'}==1 || $autoimp->{'mode'}==3 || $autoimp->{'mode'}==5 ||$autoimp->{'mode'}==7 || $autoimp->{'mode'}==9 || $autoimp->{'mode'}==11)
		{
			$self->client()->mode($channel, '+v', $name);
		}
	if ($autoimp->{'mode'}==2 || $autoimp->{'mode'}==3 || $autoimp->{'mode'}==8 ||$autoimp->{'mode'}==9)
		{
			$self->client()->mode($channel, '+h', $name);
		}
	if ($autoimp->{'mode'}==4 || $autoimp->{'mode'}==5 || $autoimp->{'mode'}==10 || $autoimp->{'mode'}==11)
		{
			$self->client()->mode($channel, '+o', $name);
		}
	if ($autoimp->{'mode'}>=7  && $autoimp->{'channel'}==$channel )
		{
			$self->client()->mode($channel, '+a', $name);
		}
}
sub irc_part
{
	my($self,$message) = @_;
	my($channel) = $message->{'channel'};
 	my $prep=$self->search_user($message);
	my $name=$prep->{'nickname'};
	my $flyby = $self->dbi()->select_record('users_misc',{'LOWER(name)' => lc($name) });
	if ((time()-('part_time')-$flyby->{'join_time'}) <=15)
		{
			$self->client()->mode($channel, '+b', $flyby->{'user'});
			$self->dbi()->delete_record('users_misc',$flyby);
		}
	else
		{
			$self->dbi()->delete_record('users_misc',$flyby);
		}
}

1;
__END__
